// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

package utils

import (
	"os"
	"regexp"
	"sort"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestCopyDir(t *testing.T) {
	src := "testdata/copy-src"
	dest := "testdata/copy-dest"

	pwd, _ := os.Getwd()
	defer func() {
		_ = os.Chdir(pwd)
	}()

	err := os.RemoveAll(dest)
	if err != nil {
		return
	}

	err = CopyDir(src, dest)
	if err != nil {
		t.Error(err)
		return
	}

	sf, _ := os.Open(src)
	defer sf.Close()
	df, _ := os.Open(dest)
	defer df.Close()

	sNames, _ := sf.Readdirnames(-1)
	dNames, _ := df.Readdirnames(-1)
	sort.Strings(sNames)
	sort.Strings(dNames)
	for i := 0; i < len(sNames); i++ {
		assert.Equal(t, sNames[i], dNames[i])
	}
}

func TestCreateFile(t *testing.T) {
	err := CreateFile("")
	assert.Error(t, err)

	file := "testdata/create-testfile"
	err = CreateFile(file)
	require.NoError(t, err)
	os.Remove(file)

	file = "testdata/xxx/create-testfile"
	err = CreateFile(file)
	assert.Error(t, err)
}

func TestSymlinkFile(t *testing.T) {
	var datas = []struct {
		src     string
		dest    string
		success bool
	}{
		{
			src:     "testdata/testfile",
			dest:    "testdata/test_symlink",
			success: true,
		},
		{
			src:     "testdata/testfile",
			dest:    "testdata/test1",
			success: false,
		},
		{
			src:     "testdata/testfile_xxx",
			dest:    "testdata/test_symlink",
			success: false,
		},
	}

	for _, data := range datas {
		if data.success {
			assert.Equal(t, SymlinkFile(data.src, data.dest), nil)
			os.Remove(data.dest)
		} else {
			assert.NotEqual(t, SymlinkFile(data.src, data.dest), nil)
		}
	}
}

func TestGetFiles(t *testing.T) {
	var datas = []struct {
		dir    string
		length int
		ret    bool
	}{
		{
			dir:    "testdata/test-get_files",
			length: 2,
			ret:    true,
		},
		{
			dir:    "testdata/xxx",
			length: 0,
			ret:    false,
		},
		{
			dir:    "testdata/testfile",
			length: 0,
			ret:    false,
		},
	}

	for _, data := range datas {
		files, err := GetFilesInDir(data.dir)
		assert.Equal(t, len(files), data.length)
		if data.ret {
			assert.Equal(t, err, nil)
		} else {
			assert.NotEqual(t, err, nil)
		}
	}
}

func TestFindFileInDir(t *testing.T) {
	var testArgs = []struct {
		dir      string
		file     string
		expected bool
	}{
		{
			dir:      "testdata",
			file:     "testfile",
			expected: true,
		},
		{
			dir:      "testdata",
			file:     "test_file",
			expected: false,
		},
		{
			dir:      "testdata",
			file:     "file1",
			expected: true,
		},
	}

	for _, arg := range testArgs {
		_, err := FindFileInDir(arg.dir, arg.file)
		if arg.expected {
			require.NoError(t, err)
		} else {
			require.Error(t, err)
		}
	}
}

func TestFindFileInDirRegexp(t *testing.T) {
	var testArgs = []struct {
		dir           string
		file          string
		expected      string
		expectedIndex int
	}{
		{
			dir:           "testdata",
			file:          `config-[0-9\.]+-.*-desktop`,
			expected:      "testdata/config-5.10.0-amd64-desktop",
			expectedIndex: 0,
		},
	}

	for _, arg := range testArgs {
		re := regexp.MustCompile(arg.file)
		matches, err := FindFileInDirRegexp(arg.dir, re)
		if arg.expectedIndex >= 0 {
			require.NoError(t, err)
			require.True(t, len(matches) > arg.expectedIndex)
			assert.Equal(t, arg.expected, matches[arg.expectedIndex])
		} else {
			require.Error(t, err)
		}
	}
}

func TestFindContentInFileRegexp(t *testing.T) {
	var testArgs = []struct {
		file          string
		content       string
		expected      string
		expectedIndex int
	}{
		{
			file:          "testdata/config-5.10.0-amd64-desktop",
			content:       `^\s*CONFIG_CPU_FREQ_DEFAULT_GOV_(\w+)`,
			expected:      "PERFORMANCE",
			expectedIndex: 1,
		},
	}

	for _, arg := range testArgs {
		re := regexp.MustCompile(arg.content)
		_, matches, err := FindContentInFileRegexp(arg.file, re)
		if arg.expectedIndex >= 0 {
			require.NoError(t, err)
			require.True(t, len(matches) > arg.expectedIndex)
			assert.Equal(t, arg.expected, matches[arg.expectedIndex])
		} else {
			require.Error(t, err)
		}
	}
}
